//---------------------------------------------------------------------------------
//							Timer Delay 
//				   Write By Mohammad mazarei @ 1390/6/17
//					Email: mohammad.mazarei@gmail.com
//					 Publish In KavirElectronic.ir
//---------------------------------------------------------------------------------
#ifndef USART_H
#define USART_H
#include <at91sam7x256.h>
#include <lib_at91sam7x256.h>
//-------------------------- Define PIO USART PIN ---------------------------------
#define USART0_PORT		AT91C_BASE_PIOA	
#define TXD0			1	
#define RXD0			0
#define USART1_PORT		AT91C_BASE_PIOA
#define TXD1			6	
#define RXD1			5


//-------------------------- Fast Macro To Use USART -------------------------------
#define USART_Configure_PIO(USART_PORT,USART_PIN)		USART_PORT->PIO_PDR|=(1<<USART_PIN);USART_PORT->PIO_MDER|=(1<<USART_PIN);USART_PORT->PIO_ASR|=(1<<USART_PIN)
#define USART0											AT91C_BASE_US0
#define USART1											AT91C_BASE_US1
/// Basic asynchronous mode, i.e. 8 bits no parity.
#define USART_MODE_ASYNCHRONOUS 						(AT91C_US_CHRL_8_BITS | AT91C_US_PAR_NONE)
/// IRDA mode
#define USART_MODE_IRDA         						(AT91C_US_USMODE_IRDA | AT91C_US_CHRL_8_BITS | AT91C_US_PAR_NONE | AT91C_US_FILTER)

/*
Key For Make Custom Mode:

 Usart mode:
 	AT91C_US_USMODE_NORMAL               Normal
 	AT91C_US_USMODE_RS485                RS485
 	AT91C_US_USMODE_HWHSH                Hardware Handshaking
 	AT91C_US_USMODE_MODEM                Modem
 	AT91C_US_USMODE_ISO7816_0            ISO7816 protocol: T = 0
 	AT91C_US_USMODE_ISO7816_1            ISO7816 protocol: T = 1
 	AT91C_US_USMODE_IRDA                 IrDA
 	AT91C_US_USMODE_SWHSH                Software Handshaking

Clock Selection (Baud Rate generator Input Clock):
 	AT91C_US_CLKS_CLOCK                Clock
 	AT91C_US_CLKS_FDIV1                fdiv1
 	AT91C_US_CLKS_SLOW                 slow_clock (ARM)
 	AT91C_US_CLKS_EXT                  External (SCK)

Character Length:
 	AT91C_US_CHRL_5_BITS               Character Length: 5 bits
 	AT91C_US_CHRL_6_BITS               Character Length: 6 bits
 	AT91C_US_CHRL_7_BITS               Character Length: 7 bits
 	AT91C_US_CHRL_8_BITS               Character Length: 8 bits
    AT91C_US_SYNC         			   Synchronous Mode Select

Number of Stop bits:
 	AT91C_US_NBSTOP_1_BIT                1 stop bit
 	AT91C_US_NBSTOP_15_BIT               Asynchronous (SYNC=0) 2 stop bits Synchronous (SYNC=1) 2 stop bits
 	AT91C_US_NBSTOP_2_BIT                2 stop bits
*/
//------------------------------------------------------------------------------
/// \dir
/// !Purpose
/// 
/// This module provides several definitions and methods for using an USART
/// peripheral.
///
/// !Usage
/// -# Enable the USART peripheral clock in the PMC.
/// -# Enable the required USART PIOs (see pio.h).
/// -# Configure the UART by calling USART_Configure.
/// -# Enable the transmitter and/or the receiver of the USART using
///    USART_SetTransmitterEnabled and USART_SetReceiverEnabled.
/// -# Send data through the USART using the USART_Write and
///    USART_WriteBuffer methods.
/// -# Receive data from the USART using the USART_Read and
///    USART_ReadBuffer functions; the availability of data can be polled
///    with USART_IsDataAvailable.
/// -# Disable the transmitter and/or the receiver of the USART with
///    USART_SetTransmitterEnabled and USART_SetReceiverEnabled.
//------------------------------------------------------------------------------

extern void USART_Configure(
    AT91S_USART *usart,
    unsigned int mode,
    unsigned int baudrate,
    unsigned int masterClock);

extern void USART_SetTransmitterEnabled(AT91S_USART *usart, unsigned char enabled);

extern void USART_SetReceiverEnabled(AT91S_USART *usart, unsigned char enabled);

extern void USART_Write(
    AT91S_USART *usart, 
    unsigned short data, 
    volatile unsigned int timeOut);

extern unsigned char USART_WriteBuffer(
    AT91S_USART *usart,
    void *buffer,
    unsigned int size);

extern unsigned short USART_Read(
    AT91S_USART *usart, 
    volatile unsigned int timeOut);

extern unsigned char USART_ReadBuffer(
    AT91S_USART *usart,
    void *buffer,
    unsigned int size);

extern unsigned char USART_IsDataAvailable(AT91S_USART *usart);

extern void USART_SetIrdaFilter(AT91S_USART *pUsart, unsigned char filter);

#endif //#ifndef USART_H

